/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include    "branch.h"
#include    <stdexcept>

namespace tree{

//-----------------------------------------------------------------------------
// Add new nodes to this branch draft

bool TBranchDraft::Add(const vector<string> &new_nodes, const string &new_attr){

    if(new_nodes.empty() && new_attr.empty()){
        throw invalid_argument("Empty branch draft nodes are not allowed");
        return false;
        }

    if(mark >= 0){
        throw logic_error("Modification of a completed branch draft "
            "is not allowed");
        return false;
        }

    nodes.insert(nodes.end(), new_nodes.begin(), new_nodes.end());

    attr = new_attr;

    return true;
    }

//-----------------------------------------------------------------------------
// Complete this branch draft by writting a branch mark of this branch and a
// branch mark of a master branch if this branch is additional

bool TBranchDraft::Complete(const int this_branch_bm,
        const int master_branch_bm, const bool enable_indexing){

    if(this_branch_bm < 0){
        throw invalid_argument("Negative branch mark is not allowed");
        return false;
        }

    mark = this_branch_bm;
    master_branch = master_branch_bm;
    index_enabled = enable_indexing;

    return true;
    }

//-----------------------------------------------------------------------------

} // namespace tree

#ifdef      LTREE_DEBUGGING
#include    "branch_debug.cpp"
#endif
