/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include    "selected_data.h"

namespace tree{

//-----------------------------------------------------------------------------
// Produce a string representation of an item of an entry

string TSelectedDataEntryItem::Stringify(const string left_border,
        const string right_border, const string delim) const{

    string res_str;
    const bool need_delim = this->size() > 1;

    for(const auto &val : *this){
        res_str += (need_delim ? left_border + val + right_border : val) +
            delim;
        }

    if(!res_str.empty()){
        res_str.pop_back();
        }

    return res_str;
    }

//-----------------------------------------------------------------------------
// Produce a string representation of an entry of a data selection

string TSelectedDataEntry::Stringify(const string left_border,
        const string right_border, const string delim,
        const bool avoid_empty_lines) const{

    string res_str;
    bool empty_item = true;

    for(const auto item : *this){
        const string item_str = item.Stringify(left_border, right_border, delim);
        if(!item_str.empty()){
            empty_item = false;
            }
        res_str += "\"" + item_str + "\",";
        }

    if(!res_str.empty()){
        res_str.pop_back();
        }

    if(avoid_empty_lines && empty_item){
        res_str.clear();
        }

    return res_str;
    }

//-----------------------------------------------------------------------------
// Push all elements of a given item into specified item of this entry

void TSelectedDataEntry::PushAll(const size_t i,
        const TSelectedDataEntryItem &src){

    if(i >= this->size()){
        return;
        }

    const size_t mem_step = src.size();

    for(size_t j = 0; j < src.size(); j++){
        PushBack(this->at(i), src.at(j), mem_step);
        }
    }

//-----------------------------------------------------------------------------
// Produce a string representation of a data selection

string TSelectedData::Stringify(const string left_border,
        const string right_border, const string delim,
        const bool avoid_empty_lines) const{

    string res_str;

    for(const auto &entry : *this){
        const string entry_str = entry.Stringify(
            left_border, right_border, delim, avoid_empty_lines
            );
        if(avoid_empty_lines && entry_str.empty()){
            continue;
            }
        res_str += entry_str + "\n";
        }

    return res_str;
    }

//-----------------------------------------------------------------------------

} // namespace tree
