/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include    "selected_values_of_additional_branches.h"

#ifdef      LTREE_DEBUGGING
#include    <iostream>
#endif

namespace tree{

//-----------------------------------------------------------------------------
// Push a new value into an object - append to a string of a last pair, or
// create a new pair if formation of a previous one is finished, or there
// are no values at all

void TTaggedValues::PushListEl(const string &str){

    if(!unit_opened){
        this->vector<pair<string, int>>::push_back({"{" + str, -1});
        unit_opened = true;
        }
    else if(!this->empty()){
        this->back().first += "," + str;
        }
    }

//-----------------------------------------------------------------------------
// Finish a formation of a current pair

void TTaggedValues::CloseList(const int tag){

    if(!this->empty()){
        this->back().first += "}";
        this->back().second = tag;
        }

    unit_opened = false;
    }

//-----------------------------------------------------------------------------
// Get values of all levels merged together

map<int, vector<string>> TSelectedValues::Align(const int master_branch,
        function<bool(int)> condition) const{

    #ifdef LTREE_DEBUGGING
    cout << "\tAligning values of a branches with master branch "
         << master_branch << endl;
    #endif

    if(this->find(master_branch) == this->end()){
        return {};
        }

    map<int, vector<string>> items;
    const auto &branch_marks_and_rest = this->at(master_branch);

    for(const auto &[branch_mark, levels_and_rest] : branch_marks_and_rest){
        for(const auto &[level, tagged_values]  : levels_and_rest){
            for(const auto &[value, c] : tagged_values){
                #ifdef LTREE_DEBUGGING
                cout << "\t\tChecking cell " << c << endl;
                #endif
                if(condition(c)){
                    #ifdef LTREE_DEBUGGING
                    cout << "\t\t- Pushing value '" << value << "' of branch "
                         << branch_mark << " found on level " << level
                         << endl;
                    #endif
                    items[branch_mark].push_back(value);
                    }
                #ifdef LTREE_DEBUGGING
                else{
                    cout << "\t\t- Couldn't pass the condition" << endl;
                    }
                #endif
                }
            }
        }

    return items;
    }

//-----------------------------------------------------------------------------

} // namespace tree

#ifdef      LTREE_DEBUGGING
#include    "selected_values_debug.cpp"
#endif
